try:
    from Products.validation.interfaces.IValidator import IValidator
    from Acquisition import aq_base
except ImportError:
    import sys, os
    sys.path.insert(0, os.path.join(os.path.dirname(__file__), os.pardir))
    from interfaces.IValidator import IValidator
    del sys, os

_marker = []

try:
    True
except NameError:
    True=1
    False=0

class MaxSizeValidator:
    __implements__ = IValidator

    def __init__(self, name, title='', description=''):
        self.name = name
        self.title = title or name
        self.description = description

    def __call__(self, value, *args, **kwargs):
        size = round(len(value) / (1024*1024)) # in MB
        maxsize=0
        instance = kwargs.get('instance', None)
        field    = kwargs.get('field', None)
        
        if kwargs.has_key('max_size'):
            maxsize = kwargs.get('max_size')
        elif hasattr(aq_base(instance), 'max_size'):
            maxsize = instance.max_size
        elif hasattr(aq_base(instance), 'getMaxSizeFor'):
            maxsize = instance.getMaxSizeFor(field.getName())
        elif hasattr(field, 'max_size'):
            maxsize = field.max_size
            
        if maxsize and size > maxsize:
            return ("Uploaded data is too large: %dMB (max %dMB)" %
                   (size, maxsize))
        else:
            return True

try:
    from DateTime import DateTime
except ImportError:
    # XXX use python datetime?
    DateTime = lambda date: None

class DateValidator:

    __implements__ = IValidator

    def __init__(self, name, title='', description=''):
        self.name = name
        self.title = title or name
        self.description = description

    def __call__(self, value, *args, **kwargs):
        if not value:
            return ("Validation Failed(%s): value is "
                    "empty or not informed (%s)." % (self.name, repr(value)))
        if not isinstance(value, DateTime):
            try:
                value = DateTime(value)
            except:
                return ("Validation Failed(%s): could not "
                        "convert %s to a date.""" % (self.name, value))


validatorList = [
    MaxSizeValidator('isMaxSize', title='', description=''),
    DateValidator('isValidDate', title='', description=''),
    ]

__all__ = ('validatorList', )

